package seis2viewer;

import java.util.ArrayList;
import java.util.Locale;

/*
 * Construct is initialized with raw nordic data to be parsed, as a long string
 */
public class EventGroup extends ArrayList<Event>
{

	EventGroup()
	{
		/* No input data */
	}

	EventGroup( String rawnordic ) throws Exception
	{
		/* Input data as a raw nordic string */
		addRaw( rawnordic, false );
	}

	EventGroup( String rawnordic, boolean includeAll ) throws Exception
	{
		/* Input data as a raw nordic string */
		addRaw( rawnordic, includeAll );
	}

	/*
	 * Method to parse a raw group of events in nordic format
	 */
	private void addRaw( String rawnordic, boolean includeAll ) throws Exception
	{
		/* Check for a minimum of 1 line */
		if( rawnordic.length() < 80 )
		{
			throw new Exception( "Nordic file is too small. Could not distinguish a single line event." );
		}

		/* Break into lines */
		String[] lines = rawnordic.split( "[\\n]+" );
		if( lines.length < 1 )
		{
			throw new Exception( "Could not distinguish event lines." );
		}

		/* Cycle through lines */
		if( includeAll )
		{
			for( int i = 0; i < lines.length; i++ )
			{
				/* Find 80 character lines terminated in 1 */
				if( lines[i].length() >= 80 && lines[i].charAt( 79 ) == '1' )
				{
					/* This seems like an event line... Parse it */
					try
					{
						Event evt = new Event( lines[i] );
						this.add( evt );
					}
					catch( Exception e )
					{
						System.err.println( "Error parsing line <" + lines[i] + "> : " + e.getMessage() );
					}
				}
			}
		}
		else
		{
			for( int i = 0; i < lines.length; i++ )
			{
				/* Find 80 character lines terminated in 1 */
				if( lines[i].length() >= 80 && lines[i].charAt( 79 ) == '1' )
				{
					/* Check if last line was empty */
					if( i > 0 && !lines[i - 1].trim().isEmpty() )
					{
						continue;
					}

					/* This seems like an event line... Parse it */
					try
					{
						Event evt = new Event( lines[i] );
						this.add( evt );
					}
					catch( Exception e )
					{
						System.err.println( "Error parsing line <" + lines[i] + "> : " + e.getMessage() );
					}
				}
			}
		}
		if( this.isEmpty() )
			throw new Exception( "Could not read events from file" );
	}

	/* Method to generate NLL raw data */
	public String toNLL()
	{
		String out = "";
		Event curEvent = null;
		for( int i = 0, m = this.size(); i < m; i++ )
		{
			out += this.get( i ).toNLL() + "\n";
		}
		return out;
	}

	/* Method to generate NLL Header grid */
	public String toNLLGrid( boolean useLocal )
	{
		/* Find origin and extent of grid */
		double xOrig = 0, yOrig = 0, zOrig = 0;
		double xMin = Double.MAX_VALUE, xMax = -Double.MAX_VALUE,
				yMin = Double.MAX_VALUE, yMax = -Double.MAX_VALUE, zMin = -Double.MAX_VALUE;
		for( int i = 0, m = this.size(); i < m; i++ )
		{
			Event cevent = this.get( i );
			//xOrig += cevent.lon;
			//yOrig += cevent.lat;
			if( cevent.lon > xMax )
				xMax = cevent.lon;
			if( cevent.lon < xMin )
				xMin = cevent.lon;
			if( cevent.lat > yMax )
				yMax = cevent.lat;
			if( cevent.lat < yMin )
				yMin = cevent.lat;
			if( cevent.dep > zMin )
				zMin = cevent.dep;
		}
		double dx = xMax - xMin;
		double dy = yMax - yMin;
		System.out.println( String.format( Locale.UK, "Grid size: %.0f degrees x %.0f degrees x %.0f km", dx, dy, zMin ) );
		xOrig = xMin - dx / 2;
		yOrig = yMin - dx / 2;
		String out = null;
		if( useLocal && dx < 8 && dy < 8 )
		{
			System.out.println("Using local mode.");
			out = String.format( Locale.UK, "%.0f %.0f %.0f 0.000 0.000  0.000 1.000 1.000 1.000 PROB_DENSITY\n"
					+ "TRANSFORM SIMPLE latOrig %7.3f longOrig %8.3f rotAngle   0.0",
					2 * getDist( xMin, yMin, 0.0, xMax, yMin, 0.0 ),
					2 * getDist( xMin, yMin, 0.0, xMin, yMax, 0.0 ),
					zMin * 1.1,
					yOrig, xOrig );
		}
		else
		{
			System.out.println("Using global mode.");
			out = String.format( Locale.UK, "%.0f %.0f %.0f 0.000 0.000  0.000 1.000 1.000 1.000 PROB_DENSITY\n"
					+ "TRANSFORM GLOBAL latOrig %7.3f longOrig %8.3f rotAngle   0.0",
					2 * getDist( xMin, yMin, 0.0, xMax, yMin, 0.0 ),
					2 * getDist( xMin, yMin, 0.0, xMin, yMax, 0.0 ),
					zMin * 1.1,
					yOrig, xOrig );
		}

		return out;
	}

	private double getDist( double x1, double y1, double z1, double x2, double y2, double z2 )
	{
		double R = 6371; // km
		double toRad = Math.PI / 180;
		double dLat = ( y2 - y1 ) * toRad;
		double dLon = ( x2 - x1 ) * toRad;
		double a = Math.sin( dLat / 2 ) * Math.sin( dLat / 2 )
				+ Math.cos( y1 * toRad ) * Math.cos( y2 * toRad )
				* Math.sin( dLon / 2 ) * Math.sin( dLon / 2 );
		double c = 2 * Math.atan2( Math.sqrt( a ), Math.sqrt( 1 - a ) );
		double d0 = R * c;
		return Math.sqrt( d0 * d0 + ( z2 - z1 ) * ( z2 - z1 ) );
	}
}
